﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Maps;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public ObservableCollection<MapLocation> Locations { get; set; }
        Random _random = new Random();

        public MainWindow()
        {
            InitializeComponent();

            Locations = new ObservableCollection<MapLocation>();
            for (int i = 0; i < 8000; i++)
                Locations.Add(new MapLocation { Latitude = _random.Next(-75, 75), Longitude = _random.Next(-180, 180) });

            this.DataContext = this;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            // show the loading indicator
            LoadingIndicator.Visibility = System.Windows.Visibility.Visible;

            LoadingSeries loadingSeries = new LoadingSeries(new GeographicSymbolSeries());
            loadingSeries.SeriesFinished += loadingSeries_SeriesFinished;
            loadingSeries.Series.ItemsSource = Locations;
            loadingSeries.Series.LongitudeMemberPath = "Longitude";
            loadingSeries.Series.LatitudeMemberPath = "Latitude";
            loadingSeries.Series.MaximumMarkers = 8000;
            loadingSeries.Series.MarkerTemplate = LayoutRoot.Resources["markerTemplate"] as DataTemplate;
            GeoMap.Series.Add(loadingSeries.Series);
        }

        void loadingSeries_SeriesFinished(object sender)
        {
            // hide the indicator.
            LoadingIndicator.Visibility = System.Windows.Visibility.Collapsed;
        }
    }

    public class LoadingSeries : INotifyPropertyChanged
    {
        // the series in the middle of loading.
        public GeographicSymbolSeries Series { get; set; }

        public delegate void SeriesFinishedHandler(object sender);
        public event SeriesFinishedHandler SeriesFinished;

        // whether the series is loading or not
        private bool _isLoading;
        public bool IsLoading
        {
            get
            {
                return _isLoading;
            }
            set
            {
                SetProperty(ref _isLoading, value, "IsLoading");
            }
        }

        public LoadingSeries(GeographicSymbolSeries newlyInstantiatedSeries)
        {
            Series = new GeographicSymbolSeries();
            Series.Loaded += SeriesLoaded;
        }

        // When the series is first instantiated, the loaded event will fire
        private void SeriesLoaded(object sender, RoutedEventArgs e)
        {
            Series.LayoutUpdated += SeriesLayoutUpdated;
            IsLoading = true;
        }

        // After the loaded event fires, the series creates all the points.  We know it will
        // finish when the layout updated event fires.
        private void SeriesLayoutUpdated(object sender, EventArgs e)
        {
            Series.LayoutUpdated -= SeriesLayoutUpdated;
            IsLoading = false;
            if (SeriesFinished != null)
                SeriesFinished(this);
        }

        #region INotifyPropertyChanged Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        protected void SetProperty<T>(ref T field, T value, string propertyName)
        {
            field = value;
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
    }

    public class MapLocation
    {
        public double Longitude { get; set; }
        public double Latitude { get; set; }
    }
}
